/*
   Vesa Bios Extensions SDK (Beta)
   ------------------------------------
   Ben Wright
   bwright@tcac.com

   Started 10/23/1995
*/

#include <VBESDK.hpp>



unsigned char far *VBEInfo::signature(void) {
   static unsigned int initialized = 0;
   static unsigned char far signature[5];

   if (!initialized) {
      initialized = -1;
      signature[0] = __signature[0];
      signature[1] = __signature[1];
      signature[2] = __signature[2];
      signature[3] = __signature[3];
      signature[4] = '\0';
      }

   return signature;
   }

unsigned int VBE::getinfo(VBEInfo far *_vbeinfo) {
   unsigned int rcode;

   asm {
      pusha; push es;

      mov ax, 0x4f00;
      les di, _vbeinfo;
      int 0x10;
      mov rcode, ax;

      pop es; popa;
      }

   return rcode;
   }

unsigned int VBE::getmodeinfo(VBEModeInfo far *_vbemodeinfo, unsigned int _modenumber) {
   unsigned int rcode;

   asm {
      pusha; push es;

      mov ax, 0x4f01;
      les di, _vbemodeinfo;
      mov cx, _modenumber;
      int 0x10;
      mov rcode, ax;

      pop es; popa;
      }

   return rcode;
   }

unsigned int VBE::setmode(unsigned int _modenumber) {
   unsigned int rcode;

   asm {
      pusha;

      mov ax, 0x4f02;
      mov bx, _modenumber;
      int 0x10;
      mov rcode, ax;

      popa;
      }

   return rcode;
   }

unsigned int VBE::getmode(unsigned int far *_modenumber) {
   unsigned int rcode;

   unsigned int modenumber;

   asm {
      pusha;

      mov ax, 0x4f03;
      int 0x10;
      mov rcode, ax;
      mov modenumber, bx;

      popa;
      }

   *_modenumber = modenumber;
   return rcode;
   }

unsigned int VBE::saverestorestate(unsigned char _subfunction, unsigned int _flags, void far *_buffer, unsigned int far *_size) {
   unsigned int rcode;

   unsigned int size;

   asm {
      pusha; push es;

      mov ax, 0x4f04;
      mov dl, _subfunction;
      mov cx, _flags;
      les di, _buffer; mov bx, di;
      int 0x10;
      mov rcode, ax;
      mov size, bx;

      pop es; popa;
      }

   *_size = size;
   return rcode;
   }

unsigned int VBE::windowcontrol(unsigned char _subfunction, unsigned char _window, unsigned int far *_windownumber) {
   unsigned int rcode;

   unsigned int windownumber = *_windownumber;

   asm {
      pusha;

      mov ax, 0x4f05;
      mov bh, _subfunction; mov bl, _window;
      mov dx, windownumber;
      int 0x10;
      mov rcode, ax;
      mov windownumber, dx;

      popa;
      }

   *_windownumber = windownumber;
   return rcode;
   }

unsigned int VBE::scanlinecontrol(unsigned char _subfunction, unsigned int _desiredwidth, unsigned int far *_actualbytes, unsigned int far *_actualpixels, unsigned int far *_maxscanlines) {
   unsigned int rcode;

   unsigned int actualbytes;
   unsigned int actualpixels;
   unsigned int maxscanlines;

   asm {
      pusha;

      mov ax, 0x4f06;
      mov bl, _subfunction;
      mov cx, _desiredwidth;
      int 0x10;
      mov rcode, ax;
      mov actualbytes, bx;
      mov actualpixels, cx;
      mov maxscanlines, dx;

      popa;
      }

   *_actualbytes = actualbytes;
   *_actualpixels = actualpixels;
   *_maxscanlines = maxscanlines;
   return rcode;
   }

unsigned int VBE::displaystart(unsigned char _subfunction, unsigned int far *_xcor, unsigned int far *_ycor) {
   unsigned int rcode;

   unsigned int xcor = *_xcor;
   unsigned int ycor = *_ycor;

   asm {
      pusha;

      mov ax, 0x4f07;
      xor bh, bh;
      mov bl, _subfunction;
      mov cx, xcor;
      mov dx, ycor;
      int 0x10;
      mov rcode, ax;
      mov xcor, cx;
      mov ycor, dx;

      popa;
      }

   *_xcor = xcor;
   *_ycor = ycor;
   return rcode;
   }

unsigned int VBE::paletteformatcontrol(unsigned char _subfunction, unsigned char _desiredbits, unsigned char far *_currentbits) {
   unsigned int rcode;

   unsigned char currentbits;

   asm {
      pusha;

      mov ax, 0x4f08;
      mov bl, _subfunction;
      mov bh, _desiredbits;
      int 0x10;
      mov rcode, ax;
      mov currentbits, bh;

      popa;
      }

   *_currentbits = currentbits;
   return rcode;
   }

unsigned int VBE::palettedatacontrol(unsigned char _subfunction, unsigned int _totalregisters, unsigned int _startregister, VBEPaletteData far *_vbepalettedata) {
   unsigned int rcode;

   asm {
      pusha; push es;

      mov ax, 0x4f09;
      mov bl, _subfunction;
      mov cx, _totalregisters;
      mov dx, _startregister;
      les di, _vbepalettedata;
      int 0x10;
      mov rcode, ax;

      pop es; popa;
      }

   return rcode;
   }

unsigned int VBE::getpmodeinterface(void far *_tableaddress, unsigned int far *_tablelength) {
   unsigned int rcode;

   unsigned int tablelength;

   asm {
      pusha; push es;

      mov ax, 0x4f0a;
      xor bl, bl;
      int 0x10;
      mov rcode, ax;
      mov [word ptr _tableaddress+2], es;
      mov [word ptr _tableaddress+0], di;

      pop es; popa;
      }

   *_tablelength = tablelength;
   return rcode;
   }
