/*
   Vesa Bios Extensions SDK (Beta)
   ------------------------------------
   Ben Wright
   bwright@tcac.com

   Started 10/23/1995
*/

#ifndef __VBESDK_HPP
#define __VBESDK_HPP

#include <Standard.hpp>



class VBEInfo {
   private:
      unsigned char __signature[4];
      unsigned int __version;
      unsigned char far *__oemstring;
      unsigned long int __capabilities;
      unsigned int far *__videomodelist;
      unsigned int __totalmemory;
      unsigned int __oemsoftwarerevision;
      unsigned char far *__oemvendorname;
      unsigned char far *__oemproductname;
      unsigned char far *__oemproductrevision;

      unsigned char __reserved[222];
      unsigned char __oemdata[256];

   public:
      VBEInfo() { initialize(); }
      void initialize(void) { __signature[0] = 'V'; __signature[1] = 'B'; __signature[2] = 'E'; __signature[3] = '2'; }

      unsigned char far *signature(void);
      unsigned int version(Version _version) { return (_version == Major) ? (__version >> 0x08) : (__version & 0xf); }
      unsigned char far *oemstring(void) { return __oemstring; }
      unsigned long int capabilities(void) { return __capabilities; }
      unsigned int videomodelist(unsigned int _modenumber) { return __videomodelist[_modenumber]; }
      unsigned int totalmemory(void) { return __totalmemory; }
      unsigned int oemsoftwarerevision(Version _version) { return (_version == Major) ? (__oemsoftwarerevision >> 0x08) : (__oemsoftwarerevision & 0xf); }
      unsigned char far *oemvendorname(void) { return __oemvendorname; }
      unsigned char far *oemproductname(void) { return __oemproductname; }
      unsigned char far *oemproductrevision(void) { return __oemproductrevision; }

   };

class VBEModeInfo {
   private:
      unsigned int __modeattributes;
      unsigned char __windowaattributes;
      unsigned char __windowbattributes;
      unsigned int __windowgranularity;
      unsigned int __windowsize;
      unsigned int __windowasegment;
      unsigned int __windowbsegment;
      void far *__windowfunction;
      unsigned int __bytesperscanline;

      unsigned int __xresolution;
      unsigned int __yresolution;
      unsigned char __xcharactersize;
      unsigned char __ycharactersize;
      unsigned char __numberofplanes;
      unsigned char __bitsperpixel;
      unsigned char __numberofbanks;
      unsigned char __memorymodel;
      unsigned char __banksize;
      unsigned char __numberofimagepages;
      unsigned char __reserved1;

      unsigned char __redmasksize;
      unsigned char __redfieldposition;
      unsigned char __greenmasksize;
      unsigned char __greenfieldposition;
      unsigned char __bluemasksize;
      unsigned char __bluefieldposition;
      unsigned char __reservedmasksize;
      unsigned char __reservedfieldposition;
      unsigned char __directcolormodeinfo;

      void far *__physicalbaseaddress;
      void far *__offscreenmemoryoffset;
      unsigned int __offscreenmemorysize;

      unsigned char __reserved2[206];

   public:
      unsigned int modeattributes(void) { return __modeattributes; }
      unsigned char windowaattributes(void) { return __windowaattributes; }
      unsigned char windowbattributes(void) { return __windowbattributes; }
      unsigned int windowgranularity(void) { return __windowgranularity; }
      unsigned int windowsize(void) { return __windowsize; }
      unsigned int windowasegment(void) { return __windowasegment; }
      unsigned int windowbsegment(void) { return __windowbsegment; }
      void far *windowfunction(void) { return __windowfunction; }
      unsigned int bytesperscanline(void) { return __bytesperscanline; }

      unsigned int xresolution(void) { return __xresolution; }
      unsigned int yresolution(void) { return __yresolution; }
      unsigned char xcharactersize(void) { return __xcharactersize; }
      unsigned char ycharactersize(void) { return __ycharactersize; }
      unsigned char numberofplanes(void) { return __numberofplanes; }
      unsigned char bitsperpixel(void) { return __bitsperpixel; }
      unsigned char numberofbanks(void) { return __numberofbanks; }
      unsigned char memorymodel(void) { return __memorymodel; }
      unsigned char banksize(void) { return __banksize; }
      unsigned char numberofimagepages(void) { return __numberofimagepages; }
      unsigned char reserved1(void) { return __reserved1; }

      unsigned char redmasksize(void) { return __redmasksize; }
      unsigned char redfieldposition(void) { return __redfieldposition; }
      unsigned char greenmasksize(void) { return __greenmasksize; }
      unsigned char greenfieldposition(void) { return __greenfieldposition; }
      unsigned char bluemasksize(void) { return __bluemasksize; }
      unsigned char bluefieldposition(void) { return __bluefieldposition; }
      unsigned char reservedmasksize(void) { return __reservedmasksize; }
      unsigned char reservedfieldposition(void) { return __reservedfieldposition; }
      unsigned char directcolormodeinfo(void) { return __directcolormodeinfo; }

      void far *physicalbaseaddress(void) { return __physicalbaseaddress; }
      void far *offscreenmemoryoffset(void) { return __offscreenmemoryoffset; }
      unsigned int offscreenmemorysize(void) { return __offscreenmemorysize; }
   };

struct VBEPaletteData {
   public:
      unsigned char alignmentbyte;
      unsigned char red;
      unsigned char green;
      unsigned char blue;
   };

class VBE {
   public:
      unsigned int getinfo(VBEInfo far *_vbeinfo);
      unsigned int getmodeinfo(VBEModeInfo far *_vbemodeinfo, unsigned int _modenumber);
      unsigned int setmode(unsigned int _modenumber);
      unsigned int getmode(unsigned int far *_modenumber);
      unsigned int saverestorestate(unsigned char _subfunction, unsigned int _flags, void far *_buffer, unsigned int far *_size);
      unsigned int windowcontrol(unsigned char _subfunction, unsigned char _window, unsigned int far *_windownumber);
      unsigned int scanlinecontrol(unsigned char _subfunction, unsigned int _desiredwidth, unsigned int far *_actualbytes, unsigned int far *_actualpixels, unsigned int far *_maxscanlines);
      unsigned int displaystart(unsigned char _subfunction, unsigned int far *_xcor, unsigned int far *_ycor);
      unsigned int paletteformatcontrol(unsigned char _subfunction, unsigned char _desiredbits, unsigned char far *_currentbits);
      unsigned int palettedatacontrol(unsigned char _subfunction, unsigned int _totalregisters, unsigned int _startregister, VBEPaletteData far *_vbepalettedata);
      unsigned int getpmodeinterface(void far *_tableaddress, unsigned int far *_tablelength);
   };

#endif
